package com.kylecorry.andromeda.exceptions

import android.content.Context
import com.kylecorry.andromeda.core.system.CurrentApp
import com.kylecorry.andromeda.core.tryOrLog
import com.kylecorry.andromeda.files.IFileSystem
import com.kylecorry.andromeda.files.LocalFileSystem

abstract class BaseExceptionHandler(
    protected val context: Context,
    private val generator: IBugReportGenerator,
    private val filename: String = "errors/error.txt",
    private val fileSystem: IFileSystem = LocalFileSystem(context)
) {

    fun bind() {
        if (!fileSystem.getFile(filename, create = false).exists()) {
            setupHandler()
        }
        handleLastException()
    }

    abstract fun handleBugReport(log: String)

    private fun handleLastException() {
        val file = fileSystem.getFile(filename, create = false)
        if (!file.exists()) {
            return
        }
        val error = fileSystem.read(filename)
        fileSystem.delete(filename)

        handleBugReport(error)
        setupHandler()
    }

    private fun setupHandler() {
        Exceptions.onUncaughtException {
            recordException(it)
            tryOrLog {
                CurrentApp.restart(context)
            }
        }
    }

    private fun recordException(throwable: Throwable) {
        val details = generator.generate(context, throwable)
        fileSystem.write(filename, details, false)
    }

}